/*
 * JBoss, Home of Professional Open Source
 * Copyright 2013, Red Hat, Inc. and/or its affiliates, and individual
 * contributors by the @authors tag. See the copyright.txt in the
 * distribution for a full listing of individual contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.securelogix.test.jms;

import java.util.logging.Logger;
import java.util.Date;
import java.util.Properties;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;


public class HQTopicTest {
    private static final Logger log = Logger.getLogger(HQTopicTest.class.getName());
    
    private static final String DEFAULT_CONNECTION_FACTORY = "jms/RemoteConnectionFactory";
    private static final String DEFAULT_DESTINATION = "jms/topic/testTopic";
    private static final String DEFAULT_MESSAGE_COUNT = "10";
    private static final String DEFAULT_USERNAME = "quickstartUser";
    private static final String DEFAULT_PASSWORD = "quickstartPwd1!";
    private static final String INITIAL_CONTEXT_FACTORY = "org.jboss.naming.remote.client.InitialContextFactory";
    private static final String PROVIDER_URL = "remote://localhost:4447";
    
    private static enum Contact {D ("Dave"), B1 ("Bob"), M ("Mark"), J ("Jacek"), C ("Colin"), S ("Stephen"), B2 ("Brian"), K ("Kevin"), D2 ("Dolores");
    	private final String name; 
    	Contact(String name) {
    		this.name = name;
    	}
    	public String getName() {
			return name;
		}
	   public static Contact getRandom() {
	        return values()[(int) (Math.random() * values().length)];
	    }
    }
    
    private static enum Subject {
    	A ("Project update"), 
    	B ("Pushed changes"), 
    	C ("Meeting scheduled"), 
    	D ("Tests running"), 
    	E ("Checking out code"), 
    	F ("Build failure"), 
    	G ("Build success"), 
    	H ("Status"), 
    	I ("Task complete");
		private final String subject; 
		Subject(String subject) {
			this.subject = subject;
		}
		public String getSubject() {
			return subject;
		}
	   public static Subject getRandom() {
	        return values()[(int) (Math.random() * values().length)];
	    }
    }	
    
    private static enum Message {
    	A ("Please validate."), 
    	B ("Validation complete."), 
    	C ("Rescheduling now."), 
    	D ("Refactoring code to fix bug."), 
    	E ("Finished the updates."), 
    	F ("No further status to report."), 
    	G ("This may be an issue."), 
    	H ("Everything seems to be working fine now."), 
    	I ("Going to call it a day and pick it up again tomorrow.");
		private final String message; 
		Message(String message) {
			this.message = message;
		}
		public String getMessage() {
			return message;
		}
	   public static Message getRandom() {
	        return values()[(int) (Math.random() * values().length)];
	    }
    }    
    
    public static void main(String[] args) throws Exception {

        ConnectionFactory connectionFactory = null;
        Connection connection = null;
        Session session = null;
        MessageProducer producer = null;
        MessageConsumer consumer = null;
        Destination destination = null;
        String messageTemplate = null;
        Context context = null;

        try {
       	
            // Set up the context for the JNDI lookup
            final Properties env = new Properties();
            env.put(Context.INITIAL_CONTEXT_FACTORY, INITIAL_CONTEXT_FACTORY);
            env.put(Context.PROVIDER_URL, System.getProperty(Context.PROVIDER_URL, PROVIDER_URL));
            env.put(Context.SECURITY_PRINCIPAL, System.getProperty("username", DEFAULT_USERNAME));
            env.put(Context.SECURITY_CREDENTIALS, System.getProperty("password", DEFAULT_PASSWORD));
            context = new InitialContext(env);

            // Perform the JNDI lookups
            String connectionFactoryString = System.getProperty("connection.factory", DEFAULT_CONNECTION_FACTORY);
            log.info("Attempting to acquire connection factory \"" + connectionFactoryString + "\"");
            connectionFactory = (ConnectionFactory) context.lookup(connectionFactoryString);
            log.info("Found connection factory \"" + connectionFactoryString + "\" in JNDI");

            String destinationString = System.getProperty("destination", DEFAULT_DESTINATION);
            log.info("Attempting to acquire destination \"" + destinationString + "\"");
            destination = (Destination) context.lookup(destinationString);
            log.info("Found destination \"" + destinationString + "\" in JNDI");

            // Create the JMS connection, session, producer, and consumer
            connection = connectionFactory.createConnection(System.getProperty("username", DEFAULT_USERNAME), System.getProperty("password", DEFAULT_PASSWORD));
            session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
            producer = session.createProducer(destination);
            consumer = session.createConsumer(destination);
            connection.start();

            int count = Integer.parseInt(System.getProperty("message.count", DEFAULT_MESSAGE_COUNT));

            // Send the specified number of messages
            
            TextMessage message = null;
            
            
            
            for (int i = 0; i < count; i++) {
            	
            	//Thread.sleep((long)(Math.random() * 1000));
            	
            	messageTemplate = "{" + 
        	    		" \"id\" : null," +
        	    		" \"sender\" : \"" + (Contact.getRandom().getName()) + "\"," +
        	    		" \"recipient\" : \"" + (Contact.getRandom().getName()) + "\"," +
        	    		" \"alert\" : " + (Math.random() < 0.5) + "," +
        	    		" \"messageNumber\" : " + (long)(Math.random() * 12347) + "," +
        	    		" \"sentDate\" : " + new Date().getTime() + "," +
        	    		" \"receiveDate\" : null," +
        	    		" \"subject\" : \"" + (Subject.getRandom().getSubject()) + "\"," +
        	    		" \"message\" : \"" + (Message.getRandom().getMessage()) + "\"}";
        	 
            	message = session.createTextMessage(messageTemplate);
                producer.send(message);                
            }
            
            // Then receive the same number of messages that were sent
            for (int i = 0; i < count; i++) {
                message = (TextMessage) consumer.receive(5000);
            	if (message != null) {
            		log.info("Received message with content " + message.getText());
            	}
            }
        } catch (Exception e) {
            log.severe(e.getMessage());
            throw e;
        } finally {
            if (context != null) {
                context.close();
            }

            // closing the connection takes care of the session, producer, and consumer
            if (connection != null) {
                connection.close();
            }
        }
    }
}
